##------------------------------------------------------------------------
##Tool Name: UVToSpeedDirection
##Source Name: UVToSpeedDirection.py
##Version: ArcGIS 9.2
##Author: Environmental Systems Research Institute, Inc.
##
##Required Arguments: Input feature layer
##                    U i.e. x component field
##                    V i.e. y component field
##                    Output feature class
##Optional Arguments: Speed Field Name
##                    Direction Field Name
##
##Description:        Calculate Speed, Direction, and GeoAngle from U and V.
##             
##------------------------------------------------------------------------

##------------------------------------------------------------------------
# Tool Validation 
# Usage: UVToSpeedDirection <Input_Feature> <U_Field> <V_Field> <Output_Feature> {Speed_Field_Name} {Direction_Field_Name} {Geographic_Angle_Field_Name}
# - U_Field: Lists only numeric fields: Set obtain from and field filter on tool parameter property page.
# - V_Field: Lists only numeric fields: Set obtain from and field filter on tool parameter property page.
# - Speed_Field_Name: Populate with 'Speed' if empty
# - Direction_Field_Name: Populate with 'Direction' if empty
# - Output_Feature: Update schema with output fields
# - Progress bar
##------------------------------------------------------------------------

import arcgisscripting, math, string
gp = arcgisscripting.create()

try:
    # Get required parameters
    inFeature = gp.GetParameterAsText(0) 
    inUField = gp.GetParameterAsText(1) 
    inVField = gp.GetParameterAsText(2) 
    outFeature = gp.GetParameterAsText(3) 
    # Get optional parameters
    inOptSpeedFieldName = gp.GetParameterAsText(4) 
    inOptDirectionFieldName = gp.GetParameterAsText(5)
            
    # Set default parameter if empty
    if (inOptSpeedFieldName.strip() == "") or inOptSpeedFieldName == "#": 
        inOptSpeedFieldName = "Speed"
    if (inOptDirectionFieldName.strip() == "") or inOptDirectionFieldName == "#": 
        inOptDirectionFieldName = "Direction"
        
    # 1.0 Copy input to output.
    # Usage: CopyFeatures <in_features> <out_feature_class> {config_keyword} {spatial_grid_1} {spatial_grid_2} {spatial_grid_3}
    gp.AddMessage("... Step 1/4: creating output featureclass")
    gp.CopyFeatures(inFeature,outFeature) 
    
    # 2.0 Add output fields if do not exist. Checking inFeatureFieldNames because I just copied the input to output.
    # Usage: AddField <in_table> <field_name> <LONG | TEXT | FLOAT | DOUBLE | SHORT | DATE | BLOB | RASTER> 
    #        {field_precision} {field_scale} {field_length} {field_alias} {NULLABLE | NON_NULLABLE} {NON_REQUIRED | REQUIRED} {field_domain}     
    gp.AddMessage("... Step 2/4: adding fields to output featureclass")    
    gp.AddField_management(outFeature, inOptSpeedFieldName, "DOUBLE", 12, 3)  
    gp.AddField_management(outFeature, inOptDirectionFieldName, "SHORT", 10)  
        
    # 3.0 Calulate inOptSpeedFieldName
    gp.AddMessage("... Step 3/4: calculating speed in field " + inOptSpeedFieldName)    
    # Usage: CalculateField <in_table> <field> <expression> {VB | PYTHON} {code_block} 
    # math.hypot( x, y) Return the Euclidean norm, sqrt(x*x + y*y). This is the length of the vector from the origin to point (x, y). 
    speedExpression = "math.hypot(!"+inUField+"!, !"+inVField+"!)"
    gp.CalculateField_management(outFeature, inOptSpeedFieldName, speedExpression, "PYTHON", "")

    # 4.0 Calulate inOptDirectionFieldName
    gp.AddMessage("... Step 4/4: calculating direction in field " + inOptDirectionFieldName)        
    # math.atan2( y, x) Return atan(y / x), in radians. The result is between -pi and pi. 
    # The vector in the plane from the origin to point (x, y) makes this angle with the positive X axis. 
    # The point of atan2() is that the signs of both inputs are known to it, so it can compute the correct quadrant for the angle. 
    # For example, atan(1) and atan2(1, 1) are both pi/4, but atan2(-1, -1) is -3*pi/4.     
    directionExpression = "round(math.degrees(math.atan2(!"+inUField+"!, !"+inVField+"!))) + 180"
    gp.CalculateField_management(outFeature, inOptDirectionFieldName, directionExpression, "PYTHON", "")    
        
except:
    gp.AddError(gp.GetMessages(2))