using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Catalog;
using ESRI.ArcGIS.DataManagementTools;
using ESRI.ArcGIS.Display;
using ESRI.ArcGIS.Framework;
using ESRI.ArcGIS.Geoprocessing;
using ESRI.ArcGIS.Geoprocessor;
using ESRI.ArcGIS.MultidimensionTools;
using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.SystemUI;

namespace WindRendererInCSharp
{   
   
    class WindRendererInCSharp
    {
        
       
        static void Main(string[] args)
        {           
            // Variables: Inputs and outputs
            string sInNCWindDir; // Input netCDF Wind Direction file
            string sInNCWindSpeed; // nput netCDF Wind Speed file
            string sOutFeatureClass;
            string sOutFeatureLayerFile;
            string sAllTimeSteps;
            bool bAllTimeSteps = false;
            string sProjectionHourIndex;
            long lProjectionHourIndex;

            //Intermediate data, layer, fieldnames
            string sNCFLWindDir = "NCFLWindDir"; //NetCDF Feature Layer
            string sNCFLWindSpeed = "NCFLWindSpeed"; //NetCDF Feature Layer
            string sFieldWindDir = "WindDir"; //values come from WindDir_SFC
            string sFieldWindSpeed = "WindSpeed"; //values come from WindSpd_SFC
            string sFieldWindSpeedT = "WindSpeedT"; //values created from WindSpeed

            // Create Geoprocessor
            Geoprocessor gp = new Geoprocessor();
            gp.OverwriteOutput = true;

            // Declare GP tools
            AddField addFieldTool = new AddField();
            AddJoin addJoinTool = new AddJoin();
            CalculateField calculateFieldTool = new CalculateField();
            CopyFeatures copyFeaturesTool = new CopyFeatures();
            Delete deleteTool = new Delete();
            MakeFeatureLayer makeFeatureLayerTool = new MakeFeatureLayer();
            MakeNetCDFFeatureLayer makeNetCDFFeatureLayerTool = new MakeNetCDFFeatureLayer();
            RemoveJoin removeJoinTool = new RemoveJoin();
            SaveToLayerFile saveToLayerFileTool = new SaveToLayerFile();

            // Get Arguments               
            // Parameters: Input NetCDF files and output featureclass and layer file.
            sInNCWindDir = (string)args.GetValue(0);
            sInNCWindSpeed = (string)args.GetValue(1);
            sOutFeatureClass = (string)args.GetValue(2);
            sOutFeatureLayerFile = (string)args.GetValue(3);

            if (!sOutFeatureLayerFile.ToLower().Contains(".lyr"))            
            {
                sOutFeatureLayerFile = sOutFeatureLayerFile + ".lyr";
            }   

        
            // Parameter: AllTimeSteps
            sAllTimeSteps = (string)args.GetValue(4);
            if (sAllTimeSteps.Equals("true"))
            {
                bAllTimeSteps = true;            
            }
            else if (sAllTimeSteps.ToLower().Equals("false")|sAllTimeSteps.Equals("#")|sAllTimeSteps.Equals(""))
            {
                bAllTimeSteps = false;   
            }


            // Parameter: ProjectionHourIndex
            sProjectionHourIndex = (string)args.GetValue(5);
            if (sProjectionHourIndex.Equals("") | sProjectionHourIndex.Equals("#"))
            {
                sProjectionHourIndex = "0";
            }
            lProjectionHourIndex = Convert.ToInt64(sProjectionHourIndex);


            // Determine output feature class type
            Boolean isOutputShapeFile = false;        
            Boolean isOutputGDBFile = false;
            Boolean isOutputMDBFile = false;


            if (sOutFeatureClass.ToLower().Contains(".shp"))
            {
                isOutputShapeFile = true;
            }
            else if (sOutFeatureClass.ToLower().Contains(".gdb"))
            {
                isOutputGDBFile = true;
            }
            else if (sOutFeatureClass.ToLower().Contains(".mdb"))
            {
                isOutputMDBFile = true;
            }

            //Specify temporary featureclass
            string sTempFeatureClass;
            if (isOutputShapeFile.Equals(true))
            {
                sTempFeatureClass = sOutFeatureClass.Substring(0, sOutFeatureClass.Length - 4) + "tmp.shp"; //Temporary fc
            }
            else 
            {
                sTempFeatureClass = sOutFeatureClass + "tmp"; //Temporary fc
            }

            // Specify dimension value
            // Don't use sProjectionHourIndex becasue it can be # or ""
            string sDimensionValue;
            sDimensionValue = "ProjectionHr " + lProjectionHourIndex.ToString(); //ProjectionHr - hardcoded dimension

            // Delete all outputs and intermediate layers if exists        
            DeleteIfExists(gp, deleteTool, sNCFLWindDir); //Delete WindDir netCDF feature layer
            DeleteIfExists(gp, deleteTool, sNCFLWindSpeed); //Delete WindSpeed netCDF feature layer
            DeleteIfExists(gp, deleteTool, sOutFeatureClass); //Delete Output feature class
            DeleteIfExists(gp, deleteTool, sTempFeatureClass); //Delete temporary output feature class
            DeleteIfExists(gp, deleteTool, sOutFeatureLayerFile); //Delete output feature layer file


            //Step 1. MakeNetCDFFeatureLayer from WindDir
            if (bAllTimeSteps)
            {
                //Create for all timesteps
                makeNetCDFFeatureLayerTool.in_netCDF_file = sInNCWindDir;
                makeNetCDFFeatureLayerTool.variable = "WindDir_SFC";
                makeNetCDFFeatureLayerTool.x_variable = "XCells";
                makeNetCDFFeatureLayerTool.y_variable = "YCells";
                makeNetCDFFeatureLayerTool.out_feature_layer = sNCFLWindDir;
                makeNetCDFFeatureLayerTool.row_dimension = "XCells;YCells;ProjectionHr";
                gp.Execute(makeNetCDFFeatureLayerTool, null);
            }
            else
            {
                //Creating based on an index
                makeNetCDFFeatureLayerTool.in_netCDF_file = sInNCWindDir;
                makeNetCDFFeatureLayerTool.variable = "WindDir_SFC";
                makeNetCDFFeatureLayerTool.x_variable = "XCells";
                makeNetCDFFeatureLayerTool.y_variable = "YCells";
                makeNetCDFFeatureLayerTool.out_feature_layer = sNCFLWindDir;
                makeNetCDFFeatureLayerTool.row_dimension = "XCells;YCells";                
                makeNetCDFFeatureLayerTool.dimension_values = sDimensionValue;
                makeNetCDFFeatureLayerTool.value_selection_method = "BY_INDEX";
                gp.Execute(makeNetCDFFeatureLayerTool, null);
            }

            gp.AddMessage("... Step 1/15: Created netCDF feature layer from wind direction");
            
             //Step 2. CopyFeature to WindDir_FC (This is the feature Output)
            copyFeaturesTool.in_features = sNCFLWindDir;
            copyFeaturesTool.out_feature_class = sOutFeatureClass;
            gp.Execute(copyFeaturesTool, null);

            //Delete WindDir netCDF feature layer
            DeleteIfExists(gp, deleteTool, sNCFLWindDir);
            gp.AddMessage("... Step 2/15: Copied wind direction netCDF feature layer to output featureclass");
            

            //Step 3. MakeNetCDFFeatureLayer from WindSpeed
            makeFeatureLayerTool = new MakeFeatureLayer();
            if (bAllTimeSteps) 
            {
                //Create for all timesteps
                makeNetCDFFeatureLayerTool.in_netCDF_file = sInNCWindSpeed;
                makeNetCDFFeatureLayerTool.variable = "WindSpd_SFC";
                makeNetCDFFeatureLayerTool.x_variable = "XCells";
                makeNetCDFFeatureLayerTool.y_variable = "YCells";
                makeNetCDFFeatureLayerTool.out_feature_layer = sNCFLWindSpeed;
                makeNetCDFFeatureLayerTool.row_dimension = "XCells;YCells;ProjectionHr";
                gp.Execute(makeNetCDFFeatureLayerTool, null);
            }
            else
            {
            //create feature layer based on a time step
                makeNetCDFFeatureLayerTool.in_netCDF_file = sInNCWindSpeed;
                makeNetCDFFeatureLayerTool.variable = "WindSpd_SFC";
                makeNetCDFFeatureLayerTool.x_variable = "XCells";
                makeNetCDFFeatureLayerTool.y_variable = "YCells";
                makeNetCDFFeatureLayerTool.out_feature_layer = sNCFLWindSpeed;
                makeNetCDFFeatureLayerTool.row_dimension = "XCells;YCells";                
                makeNetCDFFeatureLayerTool.dimension_values = sDimensionValue;
                makeNetCDFFeatureLayerTool.value_selection_method = "BY_INDEX";
                gp.Execute(makeNetCDFFeatureLayerTool, null);
            }
        
            gp.AddMessage("... Step 3/15: Created netCDF feature layer from wind speed");

            //Step 4. CopyFeature to WindSpeed_FC
            copyFeaturesTool.in_features = sNCFLWindSpeed;
            copyFeaturesTool.out_feature_class = sTempFeatureClass;
            gp.Execute(copyFeaturesTool, null);

            //Delete WindSpeed netCDF feature layer
            DeleteIfExists(gp, deleteTool, sNCFLWindSpeed);
            gp.AddMessage("... Step 4/15: Copied wind speed netCDF feature layer to a temporary featureclass");


            //Step 5. Output feature class: add fields WindDir, WIndSpeed, and WindSpeedT
            AddFieldUsingGPTool(gp, addFieldTool, sOutFeatureClass, sFieldWindDir, "LONG", 8);
            AddFieldUsingGPTool(gp, addFieldTool, sOutFeatureClass, sFieldWindSpeed, "LONG", 8);
            AddFieldUsingGPTool(gp, addFieldTool, sOutFeatureClass, sFieldWindSpeedT, "TEXT", 20);
            gp.AddMessage("... Step 5/15: Added  fields to output featureclass");

            //Step 6. WindSpeed fc (temporary feature classs): add field WindSpeed
            AddFieldUsingGPTool(gp, addFieldTool, sTempFeatureClass, sFieldWindSpeed, "LONG", 8);
            gp.AddMessage("... Step 6/15: Added  fields to temporary featureclass");

            //Step 7. WindSpeed fc: Reclassify WindSpeed_SF to WindSpeed
            //(Workaround to reclass WindSpeed_SFC to WindSpeed.)
            string sWindSpeedInMemLayer = "WindSpeedInMemLayer";
            string sSQLExpression = "";

            //7a. Less than or equal to 2.5
            if (isOutputShapeFile.Equals(true))
            {
                sSQLExpression = Convert.ToChar(34) + "WindSpd_SF" + Convert.ToChar(34) + " <= 2.5";
            }
            else if (isOutputGDBFile.Equals(true)) 
            { 
                sSQLExpression = Convert.ToChar(34) + "WindSpd_SFC" + Convert.ToChar(34) + " <= 2.5";
            }
            else if (isOutputMDBFile.Equals(true)) 
            {
                sSQLExpression = "[WindSpd_SFC]" + " <= 2.5";
            }
        
            DeleteIfExists(gp, deleteTool, sWindSpeedInMemLayer);        

            makeFeatureLayerTool.in_features = sTempFeatureClass;
            makeFeatureLayerTool.out_layer = sWindSpeedInMemLayer;
            makeFeatureLayerTool.where_clause = sSQLExpression;
            gp.Execute(makeFeatureLayerTool, null);

            calculateFieldTool.in_table = sWindSpeedInMemLayer;
            calculateFieldTool.field = sFieldWindSpeed;
            calculateFieldTool.expression = 0;
            gp.Execute(calculateFieldTool, null);
            

            // 7b. from 2.5 to 132.5
            double fMinSpeed = 2.5;
            double fMaxSpeed;
            long lAvgSpeed = 5;

            while (fMinSpeed <= 127.5)
            {
                fMaxSpeed = fMinSpeed + 5;
            
                if (isOutputShapeFile.Equals(true)) 
                {
                    sSQLExpression = Convert.ToChar(34) + "WindSpd_SF" + Convert.ToChar(34) + " > " + fMinSpeed.ToString() + " AND " + Convert.ToChar(34) + "WindSpd_SF" + Convert.ToChar(34) + " <= " + fMaxSpeed.ToString();
                }
                else if (isOutputGDBFile.Equals(true))
                {
                    sSQLExpression = Convert.ToChar(34) + "WindSpd_SFC" + Convert.ToChar(34) + " > " + fMinSpeed.ToString() + " AND " + Convert.ToChar(34) + "WindSpd_SFC" + Convert.ToChar(34) + " <= " + fMaxSpeed.ToString();
                }
                else if (isOutputMDBFile.Equals(true))
                {
                    sSQLExpression = "[WindSpd_SFC]" + " > " + fMinSpeed.ToString() + " AND " + "[WindSpd_SFC]" + " <= " + fMaxSpeed.ToString();
                }
            
                DeleteIfExists(gp, deleteTool, sWindSpeedInMemLayer);
            

                makeFeatureLayerTool = new MakeFeatureLayer();
                makeFeatureLayerTool.in_features = sTempFeatureClass;
                makeFeatureLayerTool.out_layer = sWindSpeedInMemLayer;
                makeFeatureLayerTool.where_clause = sSQLExpression;
                gp.Execute(makeFeatureLayerTool, null);

                calculateFieldTool = new CalculateField();
                calculateFieldTool.in_table = sWindSpeedInMemLayer;
                calculateFieldTool.field = sFieldWindSpeed;
                calculateFieldTool.expression = lAvgSpeed;
                gp.Execute(calculateFieldTool, null);
                

                DeleteIfExists(gp, deleteTool, sWindSpeedInMemLayer);
                
                fMinSpeed = fMinSpeed + 5;
                lAvgSpeed = lAvgSpeed + 5;
            }

            // 7c. greater than 132.5
            if (isOutputShapeFile.Equals(true))
            {
                sSQLExpression = Convert.ToChar(34) + "WindSpd_SF" + Convert.ToChar(34) + " > 132.5";
            }
            else if (isOutputGDBFile.Equals(true))
            {
                sSQLExpression = Convert.ToChar(34) + "WindSpd_SFC" + Convert.ToChar(34) + " > 132.5";
            }
            else if (isOutputMDBFile.Equals(true)) 
            {
                sSQLExpression = "[WindSpd_SFC] > 132.5";
            }
            
            DeleteIfExists(gp, deleteTool, sWindSpeedInMemLayer);
        

            makeFeatureLayerTool = new MakeFeatureLayer();
            makeFeatureLayerTool.in_features = sTempFeatureClass;
            makeFeatureLayerTool.out_layer = sWindSpeedInMemLayer;
            makeFeatureLayerTool.where_clause = sSQLExpression;
            gp.Execute(makeFeatureLayerTool, null);

            calculateFieldTool = new CalculateField();
            calculateFieldTool.in_table = sWindSpeedInMemLayer;
            calculateFieldTool.field = sFieldWindSpeed;
            calculateFieldTool.expression = 135;
            gp.Execute(calculateFieldTool, null);

            DeleteIfExists(gp, deleteTool, sWindSpeedInMemLayer);
            gp.AddMessage("... Step 7/15: Reclassified wind speed in temporary featureclass");


             //Step 8. Calculate WindDir in WindDir fc from "WindDir_FC" field
            calculateFieldTool = new CalculateField();
            calculateFieldTool.in_table = sOutFeatureClass;
            calculateFieldTool.field = sFieldWindDir;
            //depending on the output data type the field name changes. 
            if (isOutputShapeFile.Equals(true)) 
            {
                calculateFieldTool.expression = "[WindDir_SF]"; //with shp file, will be WindDir_SF
            }
            else
            {
                calculateFieldTool.expression = "[WindDir_SFC]"; //in fc, with shp file, will be WindDir_SF
            }
            gp.Execute(calculateFieldTool, null);
            gp.AddMessage("... Step 8/15: Calculated wind direction in output featureclass");

            //NOTE: Calculate WindSpeed in WindDir_fc from WindSpeed_fc. To do this,
            //need to create a feature layer from WindDir_fc (sOutFeatureClass) and then 
            //join WindSpeed_fc (sTempFeatureClass).
            //Finally calculate item from the join field.

            //Step 9. Make a FeatureLayer from WindDir
            string sWindDirInMemLayer = "WindDirInMemLayer";
            DeleteIfExists(gp, deleteTool, sWindDirInMemLayer);

            makeFeatureLayerTool = new MakeFeatureLayer();
            makeFeatureLayerTool.in_features = sOutFeatureClass;
            makeFeatureLayerTool.out_layer = sWindDirInMemLayer;
            gp.Execute(makeFeatureLayerTool, null);
            gp.AddMessage("... Step 9/15: Created in memory feature layer from output featureclass");


            //Step 10. Add join WndSpeed_fc (sTempFeatureClass) to WindDir feature layer (sWindDirInMemLayer)
            addJoinTool.in_layer_or_view = sWindDirInMemLayer;
            addJoinTool.join_table = sTempFeatureClass;
            if (isOutputShapeFile.Equals(true))
            {
                addJoinTool.in_field = "FID"; //with shp file, join ID is FID
                addJoinTool.join_field = "FID";
            }
            else
            {
                addJoinTool.in_field = "OBJECTID"; //with shp file, join ID is FID
                addJoinTool.join_field = "OBJECTID";
            }
            gp.Execute(addJoinTool, null);
            gp.AddMessage("... Step 10/15: Joined temporary windspeed fc to output featureclass");

            //Step 11. Calculate WindSpeed
            string sFCNmWindDir; 
            string sFCNmWindSpeed; 

            FileInfo fiOutFeatureClass = new FileInfo(sOutFeatureClass);
            string sFileNameOutFeatureClass = fiOutFeatureClass.Name;
            string sFileExtensionOutFeatureClass = fiOutFeatureClass.Extension;
            if (sFileExtensionOutFeatureClass.Length > 0)
            {
                sFCNmWindDir = sFileNameOutFeatureClass.Replace(sFileExtensionOutFeatureClass, "");
            }
            else
            {
                sFCNmWindDir = sFileNameOutFeatureClass;
            }

            FileInfo fiTempFeatureClass = new FileInfo(sTempFeatureClass);
            string sFileNameTempFeatureClass = fiTempFeatureClass.Name;
            string sFileExtensionTempFeatureClass = fiTempFeatureClass.Extension;
            if (sFileExtensionTempFeatureClass.Length > 0)
            {
                sFCNmWindSpeed = sFileNameTempFeatureClass.Replace(sFileExtensionTempFeatureClass, "");
            }
            else
            {
                sFCNmWindSpeed = sFileNameTempFeatureClass;
            }

            string sToField;
            string sFromField;
            string sCalcFieldExpression;
            sToField = sFCNmWindDir + "." + sFieldWindSpeed;
            sFromField = sFCNmWindSpeed + "." + sFieldWindSpeed;
            sCalcFieldExpression = "[" + sFromField + "]";
            calculateFieldTool = new CalculateField();
            calculateFieldTool.in_table = sWindDirInMemLayer;
            calculateFieldTool.field = sToField;
            calculateFieldTool.expression = sCalcFieldExpression;
            gp.Execute(calculateFieldTool, null);
            gp.AddMessage("... Step 11/15: Calculated WindSpeed in output feature class");

            //Step 12. Remove Join
            removeJoinTool.in_layer_or_view = sWindDirInMemLayer;
            removeJoinTool.join_name = sFCNmWindSpeed;
            gp.Execute(removeJoinTool, null);

            DeleteIfExists(gp, deleteTool, sWindDirInMemLayer);        
            gp.AddMessage("... Step 12/15: Removed join between temporary windspeed featureclass and output featureclass");

            //Step 13. Calculate WindSpeedT
            //13a. Equal to 0
            if (isOutputShapeFile.Equals(true) | isOutputGDBFile.Equals(true))
            {
                sSQLExpression = Convert.ToChar(34) + sFieldWindSpeed + Convert.ToChar(34) + " = 0";
            }
            else
            {
                sSQLExpression = "[" + sFieldWindSpeed + "]" + " = 0";
            }
       
            makeFeatureLayerTool = new MakeFeatureLayer();
            makeFeatureLayerTool.in_features = sOutFeatureClass;
            makeFeatureLayerTool.out_layer = sWindDirInMemLayer;
            makeFeatureLayerTool.where_clause = sSQLExpression;
            gp.Execute(makeFeatureLayerTool, null);

            sCalcFieldExpression = Convert.ToChar(34) + "Calm Wind" + Convert.ToChar(34);
            calculateFieldTool = new CalculateField();
            calculateFieldTool.in_table = sWindDirInMemLayer;
            calculateFieldTool.field = sFieldWindSpeedT;
            calculateFieldTool.expression = sCalcFieldExpression;
            gp.Execute(calculateFieldTool, null);

            DeleteIfExists(gp, deleteTool, sWindDirInMemLayer);        
        
            //13b. Greater than 0
            if (isOutputShapeFile.Equals(true) | isOutputGDBFile.Equals(true)) 
            {
                sSQLExpression = Convert.ToChar(34) + sFieldWindSpeed + Convert.ToChar(34) + " > 0";
            }
            else
            {
                sSQLExpression = "[" + sFieldWindSpeed + "]" + " > 0"; //FC use []
            }

            sCalcFieldExpression = Convert.ToChar(34) + "Wind Speed--" + Convert.ToChar(34) + Convert.ToChar(43) + "Trim([WindSpeed])" + Convert.ToChar(43) + Convert.ToChar(34) + "K" + Convert.ToChar(34);

            makeFeatureLayerTool = new MakeFeatureLayer();
            makeFeatureLayerTool.in_features = sOutFeatureClass;
            makeFeatureLayerTool.out_layer = sWindDirInMemLayer;
            makeFeatureLayerTool.where_clause = sSQLExpression;
            gp.Execute(makeFeatureLayerTool, null);

            calculateFieldTool = new CalculateField();
            calculateFieldTool.in_table = sWindDirInMemLayer;
            calculateFieldTool.field = sFieldWindSpeedT;
            calculateFieldTool.expression = sCalcFieldExpression;
            gp.Execute(calculateFieldTool, null); //often stops here for long time calculation

            DeleteIfExists(gp, deleteTool, sWindDirInMemLayer);                
            gp.AddMessage("... Step 13/15: Calculated WindSpeedT in output featureclass");


            //Step 14. Create a WindDir layer and then save it to a layer file        
            DeleteIfExists(gp, deleteTool, sWindDirInMemLayer);                
        
            makeFeatureLayerTool = new MakeFeatureLayer();
            makeFeatureLayerTool.in_features = sOutFeatureClass;
            makeFeatureLayerTool.out_layer = sWindDirInMemLayer;
            gp.Execute(makeFeatureLayerTool, null);

            saveToLayerFileTool.in_layer = sWindDirInMemLayer;
            saveToLayerFileTool.out_layer = sOutFeatureLayerFile;
            gp.Execute(saveToLayerFileTool, null);

            DeleteIfExists(gp, deleteTool, sWindDirInMemLayer);
            gp.AddMessage("... Step 14/15: Created output layer file from output featureclass");


            //======================================================
            //   Assign Unique Value Renderer
            //======================================================

            //Step 15. Set Unique Value Renderer

            //Specify symbols
            IMarkerSymbol pCalmWind = null;
            IMarkerSymbol pWS5K = null, pWS10K = null, pWS15K = null, pWS20K = null, pWS25K = null, pWS30K = null, pWS35K = null, pWS40K = null, pWS45K = null, pWS50K = null, pWS55K = null, pWS60K = null, pWS65K = null, pWS70K = null;
            IMarkerSymbol pWS75K = null, pWS80K = null, pWS85K = null, pWS90K = null, pWS95K = null, pWS100K = null, pWS105K = null, pWS110K = null, pWS115K = null, pWS120K = null, pWS125K = null, pWS130K = null, pWS135K = null; 

            IStyleGallery pStyleGallery = new StyleGalleryClass();
            IStyleGalleryStorage pStyleGalleryStorage;
            pStyleGalleryStorage = (IStyleGalleryStorage)pStyleGallery;

            //Add the Weather.style to the IStyleGalleryStorage
            string sWeatherStylePath;
            sWeatherStylePath = pStyleGalleryStorage.DefaultStylePath + "Weather.style";
            pStyleGalleryStorage.AddFile(sWeatherStylePath);

            IEnumStyleGalleryItem pStyleItems;
            pStyleItems = pStyleGallery.get_Items("Marker Symbols", sWeatherStylePath, "");
            pStyleItems.Reset();

            IStyleGalleryItem pStyleItem;
            pStyleItem = pStyleItems.Next();

            while (pStyleItem != null)
            {
                switch (pStyleItem.Name)
                {
                    case "Wind Speed--5K":
                        pWS5K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--10K":
                        pWS10K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--15K":
                        pWS15K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--20K":
                        pWS20K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--25K":
                        pWS25K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--30K":
                        pWS30K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--35K":
                        pWS35K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--40K":
                        pWS40K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--45K":
                        pWS45K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--50K":
                        pWS50K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--55K":
                        pWS55K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--60K":
                        pWS60K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--65K":
                        pWS65K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--70K":
                        pWS70K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--75K":
                        pWS75K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--80K":
                        pWS80K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--85K":
                        pWS85K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--90K":
                        pWS90K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--95K":
                        pWS95K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--100K":
                        pWS100K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--105K":
                        pWS105K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--110K":
                        pWS110K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--115K":
                        pWS115K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--120K":
                        pWS120K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--125K":
                        pWS125K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--130K":
                        pWS130K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--135K":
                        pWS135K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                }
                pStyleItem = pStyleItems.Next();
            }  


            //Add the ESRI.style to the IStyleGalleryStorage
            string sESRIStylePath;
            sESRIStylePath = pStyleGalleryStorage.DefaultStylePath + "ESRI.style";
            pStyleItems = pStyleGallery.get_Items("Marker Symbols", sESRIStylePath, "");
            pStyleItems.Reset();

            pStyleItem = pStyleItems.Next();
            while (pStyleItem != null)
            {
                switch (pStyleItem.Name)
                {
                    case "Circle 4":
                        pCalmWind = (IMarkerSymbol)pStyleItem.Item;
                        //Make the Calm Wind Symbol a little smaller
                        pCalmWind.Size = 8;
                        break;
                }
                pStyleItem = pStyleItems.Next();
            } 
        

            //Specify the unique value renderer
            IUniqueValueRenderer pUVRenderer = new UniqueValueRenderer();
            pUVRenderer.FieldCount = 1;
            pUVRenderer.set_Field(0, "WindSpeedT");
            pUVRenderer.DefaultSymbol = (ISymbol)pCalmWind;
            pUVRenderer.UseDefaultSymbol = false;

            //Adding values
            pUVRenderer.AddValue("Calm Wind", "", (ISymbol)pCalmWind);
            pUVRenderer.AddValue("Wind Speed--5K", "", (ISymbol)pWS5K);
            pUVRenderer.AddValue("Wind Speed--10K", "", (ISymbol)pWS10K);
            pUVRenderer.AddValue("Wind Speed--15K", "", (ISymbol)pWS15K);
            pUVRenderer.AddValue("Wind Speed--20K", "", (ISymbol)pWS20K);
            pUVRenderer.AddValue("Wind Speed--25K", "", (ISymbol)pWS25K);
            pUVRenderer.AddValue("Wind Speed--30K", "", (ISymbol)pWS30K);
            pUVRenderer.AddValue("Wind Speed--35K", "", (ISymbol)pWS35K);
            pUVRenderer.AddValue("Wind Speed--40K", "", (ISymbol)pWS40K);
            pUVRenderer.AddValue("Wind Speed--45K", "", (ISymbol)pWS45K);
            pUVRenderer.AddValue("Wind Speed--50K", "", (ISymbol)pWS50K);
            pUVRenderer.AddValue("Wind Speed--55K", "", (ISymbol)pWS55K);
            pUVRenderer.AddValue("Wind Speed--60K", "", (ISymbol)pWS60K);
            pUVRenderer.AddValue("Wind Speed--65K", "", (ISymbol)pWS65K);
            pUVRenderer.AddValue("Wind Speed--70K", "", (ISymbol)pWS70K);
            pUVRenderer.AddValue("Wind Speed--75K", "", (ISymbol)pWS75K);
            pUVRenderer.AddValue("Wind Speed--80K", "", (ISymbol)pWS80K);
            pUVRenderer.AddValue("Wind Speed--85K", "", (ISymbol)pWS85K);
            pUVRenderer.AddValue("Wind Speed--90K", "", (ISymbol)pWS90K);
            pUVRenderer.AddValue("Wind Speed--95K", "", (ISymbol)pWS95K);
            pUVRenderer.AddValue("Wind Speed--100K", "", (ISymbol)pWS100K);
            pUVRenderer.AddValue("Wind Speed--105K", "", (ISymbol)pWS105K);
            pUVRenderer.AddValue("Wind Speed--110K", "", (ISymbol)pWS110K);
            pUVRenderer.AddValue("Wind Speed--115K", "", (ISymbol)pWS115K);
            pUVRenderer.AddValue("Wind Speed--120K", "", (ISymbol)pWS120K);
            pUVRenderer.AddValue("Wind Speed--125K", "", (ISymbol)pWS125K);
            pUVRenderer.AddValue("Wind Speed--130K", "", (ISymbol)pWS130K);
            pUVRenderer.AddValue("Wind Speed--135K", "", (ISymbol)pWS135K);

            //Specify rotation
            IRotationRenderer pRotationRenderer;
            pRotationRenderer = (IRotationRenderer)pUVRenderer;
            pRotationRenderer.RotationField = "WindDir"; //Field Name
            pRotationRenderer.RotationType = esriSymbolRotationType.esriRotateSymbolGeographic;


            //Get the layer file
            int lNum;
            object vValue;
            IEnumGxObject pEnumGxObject;
            IGxCatalog pGxCat = new GxCatalogClass();
            IGxObject pGxObj;

            vValue = pGxCat.GetObjectFromFullName(sOutFeatureLayerFile, out lNum);
        
            //if (vValue.GetType() is IEnumGxObject)
            if (lNum > 1)
            {
                pEnumGxObject = (IEnumGxObject)vValue;
                pGxObj = pEnumGxObject.Next();
            }
            else
            {
                pGxObj = (IGxObject)vValue;
            }
        

            IGxLayer pGxLayer;
            IGeoFeatureLayer pGeoFLayer;
            pGxLayer = (IGxLayer)pGxObj;
            pGeoFLayer = (IGeoFeatureLayer)pGxLayer.Layer;

            //Update the GeoFeatureLayer and the GxLayer
            pGeoFLayer.Renderer = (IFeatureRenderer)pUVRenderer;
            pGeoFLayer.Name = "WindVelocity";
            pGxLayer.Layer = (ILayer)pGeoFLayer;
            gp.AddMessage("... Step 15/15: Assigned renderer to output layer file");

            //Delete temporary output feature class
            DeleteIfExists(gp, deleteTool, sTempFeatureClass);
        
        }
        private static void AddFieldUsingGPTool(Geoprocessor gp, AddField addFieldTool, string sInTable, string sFieldName, string sFieldType, int lFieldLength)
        {
            addFieldTool = new AddField();
            addFieldTool.in_table = sInTable;
            addFieldTool.field_name = sFieldName;
            addFieldTool.field_type = sFieldType;
            if (sFieldType.ToUpper().Equals("LONG"))
            {
                addFieldTool.field_precision = lFieldLength;
            }
            else if (sFieldType.ToUpper().Equals("TEXT"))
            {
                addFieldTool.field_length = lFieldLength;
            }
            gp.Execute(addFieldTool, null);
        }

        private static void DeleteIfExists(Geoprocessor gp, Delete deleteTool, string sName)
        {
            object x = null;
            if (gp.Exists(sName,ref x))
            {
                deleteTool.in_data = sName;
                gp.Execute(deleteTool, null);
            }

        }
    }
}
